# Copyright 2013 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.types import (
    ActionMap,
    ResourceAttributeMap,
    SubResourceAttributeMap,
)

SERVICE_ATTR = 'service_type'
PLUGIN_ATTR = 'plugin'
DRIVER_ATTR = 'driver'

ALIAS = 'service-type'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'Neutron Service Type Management'
API_PREFIX = ''
DESCRIPTION = ("API for retrieving service providers for "
               "Neutron advanced services")
UPDATED_TIMESTAMP = '2013-01-20T00:00:00-00:00'
RESOURCE_NAME = 'service_provider'
COLLECTION_NAME = RESOURCE_NAME + 's'
RESOURCE_ATTRIBUTE_MAP: ResourceAttributeMap = {
    COLLECTION_NAME: {
        SERVICE_ATTR: {
            'allow_post': False,
            'allow_put': False,
            'is_visible': True
        },
        'name': {
            'allow_post': False,
            'allow_put': False,
            'is_visible': True
        },
        'default': {
            'allow_post': False,
            'allow_put': False,
            'is_visible': True
        }
    }
}
SUB_RESOURCE_ATTRIBUTE_MAP: SubResourceAttributeMap = {}
ACTION_MAP: ActionMap = {}
REQUIRED_EXTENSIONS = []
OPTIONAL_EXTENSIONS = []
ACTION_STATUS = {}



#  Copyright 2025 Samsung SDS. All Rights Reserved
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
#  implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from neutron_lib.api.definitions import network_ip_availability
from neutron_lib.types import (
    ActionMap,
    ResourceAttributeMap,
    SubResourceAttributeMap,
)


IP_AVAILABILITY_DETAILS = "ip_availability_details"

ALIAS = 'network-ip-availability-details'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'Network IP Availability Details Extension'
DESCRIPTION = ('Add ip_availability_details attribute to'
               ' network-ip-availabilities resource.')
UPDATED_TIMESTAMP = '2025-10-31T00:00:00-00:00'

RESOURCE_ATTRIBUTE_MAP: ResourceAttributeMap = {
    network_ip_availability.RESOURCE_PLURAL: {
        IP_AVAILABILITY_DETAILS: {
            'allow_post': False, 'allow_put': False,
            'is_visible': True
        }
    }
}
SUB_RESOURCE_ATTRIBUTE_MAP: SubResourceAttributeMap = {}
ACTION_MAP: ActionMap = {}
REQUIRED_EXTENSIONS = [network_ip_availability.ALIAS]
OPTIONAL_EXTENSIONS = []
ACTION_STATUS = {}



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from collections.abc import Callable, Iterable
from typing import Literal, TypedDict, TypeAlias


AttributeValidator = TypedDict(
    'AttributeValidator',
    {
        'type:allowed_address_pairs': None,
        'type:availability_zone_hint_list': None,
        'type:boolean': None,
        'type:dict': dict[str, object] | None,
        'type:dict_or_none': dict[str, object] | None,
        'type:dict_or_empty': dict[str, object] | None,
        'type:dict_or_nodata': dict[str, object] | None,
        'type:dns_domain_name': int | None,
        'type:dns_host_name': int | None,
        'type:fip_dns_host_name': int | None,
        'type:fixed_ips': None,
        'type:hostroutes': None,
        'type:ip_address': None,
        'type:ip_address_or_none': None,
        'type:ip_pools': None,
        'type:ip_or_subnet_or_none': None,
        'type:list_of_any_key_specs_or_none': list[dict[str, object]] | None,
        'type:list_of_dict_or_nodata': dict[str, object] | None,
        'type:list_of_regex_or_none': str | None,
        'type:list_of_subnet_service_types': None,
        'type:list_of_subnets_or_none': None,
        'type:mac_address': None,
        'type:nameservers': None,
        'type:name_string': int | None,
        'type:name_string_or_none': int | None,
        'type:name_string_not_default': int | None,
        'type:network_segments': None,
        'type:not_empty_name_string': None,
        'type:non_negative': None,
        'type:port_range': tuple[int, int] | None,
        'type:range': tuple[int, int],
        'type:range_or_none': tuple[int, int],
        'type:regex_or_none': str | None,
        'type:service_plugin_type': None,
        'type:string': int | None,
        'type:string_or_none': int | None,
        'type:subnetpool_id_or_none': None,
        'type:subnet': None,
        'type:subnet_list': None,
        'type:subnet_or_none': None,
        'type:subports': None,
        'type:uuid': None,
        'type:uuid_list': None,
        'type:uuid_list_non_empty': None,
        'type:uuid_or_none': None,
        'type:values': Iterable[object],
    },
    total=False,
)


class ResourceAttributeMapItem(TypedDict, total=False):
    allow_post: bool
    allow_put: bool
    convert_to: Callable[[object], object]
    convert_list_to: Callable[[object], object]
    default: object
    default_overrides_none: bool
    dict_populate_defaults: bool
    enforce_policy: bool
    is_visible: bool
    is_filter: bool
    is_sort_key: bool
    required_by_policy: bool
    primary_key: bool
    validate: AttributeValidator


ResourceAttributeMap: TypeAlias = dict[
    str, dict[str, ResourceAttributeMapItem]
]


class SubResourceParent(TypedDict):
    collection_name: str
    member_name: str


class SubResourceAttributeMapItem(TypedDict, total=False):
    parent: SubResourceParent
    parameters: dict[str, ResourceAttributeMapItem]


SubResourceAttributeMap: TypeAlias = dict[str, SubResourceAttributeMapItem]

ActionMap: TypeAlias = dict[
    str, dict[str, Literal['PUT', 'GET']]
]



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from neutron_lib.api.definitions import taas
from neutron_lib.api.definitions import tap_mirror
from neutron_lib.types import ActionMap, ResourceAttributeMap


ALIAS = 'tap-mirror-both-direction'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = "Tap as a Service mirror with BOTH direction support"
DESCRIPTION = ("Neutron Tap as a Service extension mirroring with support "
               "for BOTH direction.")
UPDATED_TIMESTAMP = "2025-12-15T11:45:00-00:00"

RESOURCE_ATTRIBUTE_MAP: ResourceAttributeMap = {
    tap_mirror.COLLECTION_NAME: {
        'directions': {
            'allow_post': True, 'allow_put': False,
            'validate': {
                'type:dict': {
                    'IN': {
                        'type:integer': None, 'default': None,
                        'required': False},
                    'OUT': {
                        'type:integer': None, 'default': None,
                        'required': False},
                    taas.DIRECTION_BOTH: {
                        'type:integer': None, 'default': None,
                        'required': False},
                }
            },
            'is_visible': True},
    }
}

SUB_RESOURCE_ATTRIBUTE_MAP = None
ACTION_MAP: ActionMap = {}
ACTION_STATUS = {}
REQUIRED_EXTENSIONS = [tap_mirror.ALIAS]
OPTIONAL_EXTENSIONS = []



# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api import converters
from neutron_lib.api.definitions import network
from neutron_lib.api.definitions import port
from neutron_lib import constants
from neutron_lib.types import (
    ActionMap,
    ResourceAttributeMap,
    SubResourceAttributeMap,
)


DEFAULT_PORT_SECURITY = True

PORTSECURITY = 'port_security_enabled'

# The alias of the extension.
ALIAS = 'port-security'

# Whether or not this extension is simply signaling behavior to the user
# or it actively modifies the attribute map.
IS_SHIM_EXTENSION = False

# Whether the extension is marking the adoption of standardattr model for
# legacy resources, or introducing new standardattr attributes. False or
# None if the standardattr model is adopted since the introduction of
# resource extension.
# If this is True, the alias for the extension should be prefixed with
# 'standard-attr-'.
IS_STANDARD_ATTR_EXTENSION = False

# The name of the extension.
NAME = 'Port Security'

# A prefix for API resources. An empty prefix means that the API is going
# to be exposed at the v2/ level as any other core resource.
API_PREFIX = ''

# The description of the extension.
DESCRIPTION = "Provides port security"

# A timestamp of when the extension was introduced.
UPDATED_TIMESTAMP = "2012-07-23T10:00:00-00:00"


RESOURCE_ATTRIBUTE_MAP: ResourceAttributeMap = {
    network.COLLECTION_NAME: {
        PORTSECURITY: {'allow_post': True, 'allow_put': True,
                       'convert_to': converters.convert_to_boolean,
                       'enforce_policy': True,
                       'default': DEFAULT_PORT_SECURITY,
                       'is_visible': True},
    },
    port.COLLECTION_NAME: {
        PORTSECURITY: {'allow_post': True, 'allow_put': True,
                       'convert_to': converters.convert_to_boolean,
                       'default': constants.ATTR_NOT_SPECIFIED,
                       'enforce_policy': True,
                       'is_visible': True},
    }
}

# The subresource attribute map for the extension. It adds child resources
# to main extension's resource. The subresource map must have a parent and
# a parameters entry. If an extension does not need such a map, None can
# be specified (mandatory).
SUB_RESOURCE_ATTRIBUTE_MAP: SubResourceAttributeMap = {}

# The action map: it associates verbs with methods to be performed on
# the API resource.
ACTION_MAP: ActionMap = {}

# The action status.
ACTION_STATUS = {
}

# The list of required extensions.
REQUIRED_EXTENSIONS = []

# The list of optional extensions.
OPTIONAL_EXTENSIONS = []



# Copyright (c) 2026 Red Hat Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

PVLAN = "pvlan"
PVLAN_COMMUNITY = "pvlan_community"
PVLAN_TYPE = "pvlan_type"

ISOLATED_TYPE = 'isolated'
PROMISCUOUS_TYPE = 'promiscuous'
COMMUNITY_TYPE = 'community'
COMMUNITY_NAME_MAX_LEN = 232

PVLAN_TYPES = [ISOLATED_TYPE, PROMISCUOUS_TYPE, COMMUNITY_TYPE]

COMMUNITY_NAME_REGEX = (
    "[a-zA-Z_.][a-zA-Z_.0-9]{0,%d}$" % (COMMUNITY_NAME_MAX_LEN - 1)
)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import availability_zone as az
from neutron_lib.types import (
    ActionMap,
    ResourceAttributeMap,
    SubResourceAttributeMap,
)

ALIAS = 'availability_zone_filter'
IS_SHIM_EXTENSION = True
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'Availability Zone Filter Extension'
DESCRIPTION = 'Add filter parameters to AvailabilityZone resource'
UPDATED_TIMESTAMP = '2018-06-22T10:00:00-00:00'
RESOURCE_ATTRIBUTE_MAP: ResourceAttributeMap = {}
SUB_RESOURCE_ATTRIBUTE_MAP: SubResourceAttributeMap = {}
ACTION_MAP: ActionMap = {}
REQUIRED_EXTENSIONS = [
    az.ALIAS
]
OPTIONAL_EXTENSIONS = []
ACTION_STATUS = {}
